{******************************************************************}
{*     Conduction unidimensionnelle en rgime instationnaire      *}
{*                       Mthode Implicite                        *}
{*                       Par Lad MESSAOUDI                       *}
{******************************************************************}

(* T(i,n) = -L T(i-1,n+1) + (1+2L) T(i,n+1) - L T(i+1,n+1)
   T(i,0)   = 1             (C.I)
   T(0,n+1) = 0             (C.L)
   T(1,n+1) = 0                                          *)

{  La procdure TRIDIAG permet de rsoudre le systme suivant:     }

{==================================================================}
{   B1   C1    0                 0        T1  n+1    R1  n   }
{                                                            }
{   A2   B2    C2    0                    T2         R2      }
{                                                            }
{   0    A3    B3    C3         0                            }
{                                      .        =            }
{                                                            }
{                                                            }
{                An-1   Bn-1   Cn-1       Tn-1       Rn-1    }
{                                                            }
{   0             0     An     Bn         Tn         Rn      }
{==================================================================}
PROGRAM MiniProjet;

USES Crt;

CONST    Np = 20;
         Alfa = 0;
         Beta = 0;
         Gama = 1;

TYPE
    RealArrayNp = Array[1..Np] Of Real;

VAR
   DeltaX, DeltaT, Temps   : Real;
   A, B, C, T, R           : ^RealArrayNp;
   k, N                    : Integer;
   Lambda                  : Real;

{===========================================================================}
PROCEDURE INITIALISATION_TABLEAUX;

Begin
     For k:=1 To N   Do
         Begin
              B^[k] := 0;
              A^[k] := 0;
              C^[k] := 0;
              T^[k] := 0;
              R^[k] := 0;
         End;
End;

{===========================================================================}
PROCEDURE DONNEES;

Begin
     ClrScr;
     Write('Donner l''ordre N de la matrice : '); ReadLn(N);
     Write('Donner le pas de temps : ');          ReadLn(DeltaT);
     Write('Donner le pas d''espace : ');         ReadLn(DeltaX);

     Lambda := DeltaT/(DeltaX*DeltaX);

     For k:=1 To N   Do
              Begin
                   B^[k] := 1 + 2*Lambda;
                   A^[k] := -Lambda;
                   C^[k] := -Lambda;
              End;
End;

{===========================================================================}
PROCEDURE CONDITIONS_AUX_LIMITES;

Begin
     If Temps = 0 Then                               { Conditions initiales }
        Begin
             For k:=2 To N-1 Do R^[k] := Gama;
             R^[1] := Gama + Lambda*Alfa;
             R^[N] := Gama + Lambda*Beta;
        End;
     If Temps > 0 Then
        Begin
             R^[1] := T^[1] + Lambda*Alfa;
             R^[N] := T^[N] + Lambda*Beta;
        End;
End;

{===========================================================================}
PROCEDURE TRIDIAG (Var A, B, C, T, R : RealArrayNp; N : Integer);

Var
   j     : Integer;
   Beta  : Real;
   Gamma : ^RealArrayNp;

Begin
     New(Gamma);
     If B[1] = 0 Then
                  Begin
                       WriteLn('Pause 1 dans TRIDIAG');
                       ReadLn;
                  End;
     Beta := B[1];
     T[1] := R[1]/Beta;

     For j:=2 To N Do
         Begin
              Gamma^[j] := C[j-1]/Beta;
              Beta := B[j] - A[j]*Gamma^[j];
              If Beta = 0 Then
                          Begin
                               WriteLn('Pause 2 dans TRIDIAG');
                               ReadLn;
                          End;
              T[j] := (R[j] - A[j]*T[j-1])/Beta;
         End;
     For j:=N-1 DownTo 1 Do
                T[j] := T[j] - Gamma^[j+1]*T[j+1];
     Dispose(Gamma);
End;

{===========================================================================}
PROCEDURE Affichage;

Begin
     If Temps > 0 Then
        Begin
             WriteLn;
             Write(Temps:2:4,'');Write(' 0.000');
             For k:=1 To N Do Write(T^[k]:6:3);
             Write(' 0.000');
        End;
End;

{===========================================================================}
{======================== Programme Principal ==============================}

BEGIN
    New(A); New(B); New(C); New(T); New(R);
    Initialisation_Tableaux;
    Donnees;
    ClrScr;
    WriteLn;
    WriteLn(' Temps                             Tempratures');
    Temps := 0;
    Conditions_Aux_Limites;
    Tridiag(A^, B^, C^, T^, R^, N);
    Repeat
          Affichage;
          Temps := Temps + DeltaT;
          For k:=1 To N Do R^[k] := T^[k];
          Conditions_Aux_Limites;
          Tridiag(A^, B^, C^, T^, R^, N);
    Until(Temps > 0.075);                {(T^[Round((n+1)/2)] >= 0.95);}
    ReadLn;
    Dispose(R); Dispose(T); Dispose(C); Dispose(B); Dispose(A);
End.