UNIT IMANE2D;

INTERFACE

 Uses Crt, Dos, Printer, Graph;

CONST
     F1  = #59;                        { codes des touches de fonctions }
     F2  = #60;
     F3  = #61;
     F4  = #62;
     F5  = #63;
     F6  = #64;
     F7  = #65;
     F8  = #66;
     F9  = #67;
     F10 = #68;

     FlchDebut   = #71;                { touches spciales du pav numrique }
     FlchHaut    = #72;
     FlchPgHaut  = #73;
     FlchGauche  = #75;
     FlchDroite  = #77;
     FlchFin     = #79;
     FlchBas     = #80;
     FlchPgBas   = #81;
     TouchAnnl   = #83;
     ToucheIns   = #82;
     ToucheCorr  = #8;

     CarNul      = #0;
     CarEsc      = #27;
     CarRet      = #13;


     Noir      = 0;             { les seize couleurs des cartes CGA et EGA }
     Bleu      = 1;
     Vert      = 2;
     BleuCiel  = 3;
     Rouge     = 4;
     Violet    = 5;
     Brun      = 6;
     GrisClair = 7;
     Gris      = 8;
     BleuClair = 9;
     VertClair = 10;
     BleuPale  = 11;
     Orange    = 12;
     Rose      = 13;
     Jaune     = 14;
     Blanc     = 15;

     ShiftDroitMsk  = 1;             { masques de test des touches spciales }
     ShiftGaucheMsk = 2;
     CtrlMsk        = 4;
     AltShiftMsk    = 8;
     ScrollLockMsk  = 16;
     NumLockMsk     = 32;
     CapsLockMsk    = 64;
     InsertLMsk     = 128;

     NbMaxElem = 300;               { Nombre d'lments d'un tableau rel }

VAR
     GraphDriver, GraphMode          : Integer;
     XgFen, XdFen, YbFen, YhFen      : Real;
     XgClot, XdClot, YbClot, YhClot  : Integer;
     XP1, YP1, XP2, YP2              : Real;
     XOrig, YOrig                    : Real;
     XRapport, YRapport              : Real;
     Ferme, Ouvert                   : String [6];
     MaxX, MaxY                      : Integer;
     NbreCouleurs                    : Byte;

     TYPE TableauReel = Array [1..NbMaxElem] Of Real;

Procedure EcranGraphique;
Procedure EcranTexte;
Procedure EffaceEcran;
Procedure Fenetre (F1, F2, F3, F4 : Real);

Procedure Cloture (C1, C2, C3, C4 : Integer);
Procedure PleineCloture;
Procedure Couleur (Coul: Byte);
Procedure Decoupage (X1, Y1, X2, Y2 : Real);
Procedure TraceVers (X, Y : Real);
Procedure DeplaceEn (X, Y : Real);
Procedure Bordure (Coul : Byte);
Procedure TraceBordure;
Procedure EffaceBordure;
Procedure Axes;
Procedure CorrectionAxeX (XOrig, XgFen, UnitX : Real; Var CorrectX : Real);
Procedure CorrectionAxeY (YOrig, YbFen, UnitY : Real; Var CorrectY : Real);
Procedure Gradue (UnitX, UnitY : Real);
Procedure Grille (UnitX, UnitY : Real);
Procedure Polygone (X, Y: TableauReel; Lim: Integer; Mode : String);
Procedure DessinePolygone (X, Y : TableauReel; Lim : Integer; Mode : String);
Procedure EffacePolygone (X, Y : TableauReel; Lim: Integer; Mode: String);
Procedure TraceCercle (XC, YC, R : Real; Pointille : Boolean);
Procedure Cercle (XC, YC, R:Real);
Procedure CerclePointille (XC, YC, R: Real);
Procedure Attente;
Procedure Bip;
Function Signe (N : Real): Real;
Function Puissance (Base, Expo : Real) : Real;
Function Factorielle (X : Integer) : Real;

IMPLEMENTATION

{===========================================================================}
Procedure EcranGraphique;

Var   ErrorCode : Integer;

Begin
   GraphDriver := Detect;
   InitGraph (GraphDriver, GraphMode, 'C:\TP7\BGI');
   ErrorCode := GraphResult;
   if ErrorCode <> GrOk Then
      Begin
        WriteLn ('Erreur graphique : ',GraphErrorMsg(ErrorCode));
        Halt (1)
      End;
   MaxX := GetMaxX;
   MaxY := GetMaxY;
   NbreCouleurs := GetMaxColor;
   SetColor (Blanc)
 End;

{===========================================================================}
Procedure EcranTexte;

Begin
     CloseGraph;
End;

{===========================================================================}
Procedure EffaceEcran;

Begin
   ClearDevice
End;

{===========================================================================}
Procedure Fenetre (F1, F2, F3, F4 : Real);

Begin
    XgFen := F1;
    XdFen := F2;
    YbFen := F3;
    YhFen := F4
End;

{===========================================================================}
Procedure Cloture (C1, C2, C3, C4 : Integer);
 begin
     XgClot   := C1;
     XdClot   := C2;
     YbClot   := C3;
     YhClot   := C4;
     XRapport := (XdClot - XgClot) / (xdFen - XgFen);
     YRapport := (YhClot - YbClot) / (YhFen - YbFen);
    SetViewport (XgClot, MaxY-YhClot, XdClot, MaxY-YbClot, ClipOn);
End;

{===========================================================================}
procedure PleineCloture;
begin
   Cloture (0, MaxX, 0, MaxY)
end;
{===========================================================================}
Procedure Couleur (Coul: Byte);

Begin
    SetColor (Coul);
End;


{===========================================================================}
Procedure Decoupage (X1, Y1, X2, Y2 : Real);

Type  Region = (Gauche, Droite, Basse, Haute);
      Code   = Set Of Region;

Var   C, C1, C2  : Code;
      X, Y       : Real;
      XX1, YY1   : Integer;
      XX2, YY2   : Integer;

      {==============================================}
       Procedure CodeBinaire (X, Y: Real; Var C: Code);

       Begin
            C:= [];
            If X < XgFen Then C := [Gauche]
                            Else If X > XdFen Then C := [Droite];
            If Y < YbFen Then C := C +[Basse]
                            Else If Y > YhFen Then C := C + [Haute];
       End;

Begin
    CodeBinaire (X1, Y1, C1);
    CodeBinaire (X2, Y2, C2);
    While (C1 <> []) Or (C2 <> []) Do
     Begin
      If (C1*C2) <> [] Then Exit;
       If C1 = [] Then C := C2 Else C := C1;
       If Gauche in C
       Then
         Begin
           X := XgFen;
           Y := Y1 + (Y2 - Y1) * (XgFen - X1) / (X2 - X1);
         End
      Else If Droite in C
           Then
             Begin
               X := XdFen;
               Y := Y1 + (Y2 - Y1)* (XdFen - X1)/ ( X2 - X1);
             End
           Else If Basse in C
              Then
                Begin
                  Y := YbFen;
                  X := X1 + (X2 - X1) * (YbFen - Y1) / (Y2 - Y1);
               End
              Else if Haute in C
                  Then
                       Begin
                                Y := YhFen;
                                X := X1 + (X2-X1)*(YhFen-Y1)/(Y2-Y1);
                       End
      Else if C = C1
        Then Begin
                  X1 := X;
                  Y1 := Y;
              CodeBinaire (X, Y, C1);
            End
         Else
             Begin
                  X2 := X;
                  Y2 := Y;
               CodeBinaire (X, Y, C2)
             End;
     End;
   XX1 := Round ((X1 - XgFen) * XRapport);
   YY1 := Round ((YhFen - Y1) * YRapport);
   XX2 := Round ((X2 - XgFen) * XRapport);
   YY2 := Round ((YhFen - Y2) * YRapport);

      MoveTo (XX1, YY1);
      LineTo (XX2, YY2)
 End;

{===========================================================================}
Procedure TraceVers (X, Y : Real);

Begin
     XP2 := X;
     YP2 := Y;
     Decoupage (XP1, YP1, XP2, YP2);
     XP1 := XP2;
     YP1 := YP2
End;
{===========================================================================}
Procedure DeplaceEn (X, Y : Real);

Begin
    XP1 := X;
    YP1 := Y;
    TraceVers (X, Y)
End;

{===========================================================================}
Procedure Bordure (Coul : Byte);

Begin
     SetColor (Coul);
     DeplaceEn (XgFen,YbFen);
     TraceVers (XdFen,YbFen);
     TraceVers (XdFen,YhFen);
     TraceVers (XgFen,YhFen);
     TraceVers (XgFen,YbFen);
End;

{===========================================================================}
Procedure TraceBordure;

Begin
     Bordure (Blanc);
End;

{===========================================================================}
Procedure EffaceBordure;

Begin
     Bordure (Noir);
End;

{===========================================================================}
Procedure Axes;

Const    dX = 5;
         dY = 4;
        {====================================================}
        Procedure PointeDeFlecheSurAxeX;

        Begin
             MoveRel (-dX, dY);
             LineRel ( dX,-dY);
             MoveRel (-dX,-dY);
             LineRel ( dX, dY)
        End;
        {===============================================}
        Procedure PointeDeFlecheSurAxeY;

        Begin
             MoveRel (-dY, dX);
             LineRel ( dY,-dX);
             MoveRel ( dY, dx);
             LineRel (-dY,-dX)
        End;

Begin
     if (XgFen < 0) And (XdFen > 0) Then
                    XOrig := 0
               Else Xorig := XgFen;
     if (YbFen < 0) And (YhFen > 0) Then
                    YOrig := 0
               Else YOrig := YbFen;
     DeplaceEn (XgFen, YOrig);
     TraceVers (XdFen, YOrig);
     PointeDeFlecheSurAxeX;
     DeplaceEn (XOrig, YbFen);
     Tracevers (XOrig, YhFen);
     PointeDeFlecheSurAxeY;
End;

{===========================================================================}
Procedure CorrectionAxeX (XOrig, XgFen, UnitX : Real; Var CorrectX : Real);

Var   NbreTirets : Real;

Begin
     if XOrig = 0 Then
       Begin
            NbreTirets := (XOrig - XgFen) / UnitX;
            CorrectX   := (NbreTirets - Int (NbreTirets)) * UnitX;
      End
     Else if XOrig > 0 Then
                   CorrectX := Trunc (XOrig/UnitX+1) * UnitX - XOrig
              Else CorrectX := Abs (XOrig) + Trunc(XOrig/UnitX) * UnitX;
End;

{===========================================================================}
Procedure CorrectionAxeY (YOrig, YbFen,UnitY : Real ; Var CorrectY : Real);

Var   NbreTirets : Real;

Begin;
      if YOrig = 0 Then
       Begin
            NbreTirets := (YOrig - YbFen) / UnitY;
            CorrectY   := (NbreTirets - Int (NbreTirets)) * UnitY;
       End
    Else if YOrig > 0 Then
                      CorrectY := Trunc (YOrig/UnitY+1) * UnitY - YOrig
                 Else CorrectY := Abs(YOrig) + Trunc(YOrig/UnitY) * UnitY;
End;

{===========================================================================}
Procedure Gradue (UnitX, UnitY : Real);

Var  CorrectX, CorrectY : Real;
     X, Y               : Real;
     TiretX, TiretY     : Byte;

Begin
     TiretX := 3;
     TiretY := 4;

   if UnitX > 0 Then
     Begin
          CorrectionAxeX (XOrig, XgFen, UnitX, CorrectX);
          X := XgFen + CorrectX;
       Repeat
             DeplaceEn (X, YOrig);
             TraceVers (X, YOrig);
             MoveRel (0, TiretX);
             LineRel (0, -2*TiretX);
             X := X + UnitX;
       Until X > XdFen
     End;

   if UnitY > 0 Then
      Begin
           CorrectionAxeY (YOrig, YbFen, UnitY, CorrectY);
           Y := YbFen + CorrectY;
         Repeat
               DeplaceEn (XOrig, Y);
               TraceVers (XOrig, Y);
               MoveRel (TiretY, 0);
               LineRel (-2*TiretY, 0);
               Y := Y + UnitY;
         Until Y > YhFen
       End;
End;

{===========================================================================}
Procedure Grille (UnitX, UnitY : Real);

Var   CorrectX, CorrectY, i, j : Real;

Begin
     CorrectionAxeX (XOrig, XgFen, UnitX, CorrectX);
     CorrectionAxeY (YOrig, YbFen, UnitY, CorrectY);
      I := Trunc(XgFen + CorrectX);
   While i<= Trunc(XdFen) Do
   Begin
       j := Trunc(YbFen + CorrectY);
       While j <= Trunc(YhFen) Do
       Begin
            DeplaceEn (i, j);
            TraceVers (i, j);
            j := j + UnitY;
       End;
       i := i + UnitX;
    End;
 End;

{===========================================================================}
Procedure Polygone (X, Y : TableauReel; Lim : Integer; Mode : String);

Var      I : integer;

Begin
     DeplaceEn (X[1], Y[1]);
     For i := 2 To Lim Do   TraceVers (X[i], Y[i]);
     If Mode = Ferme Then TraceVers (X[1], Y[1]);
End;

{===========================================================================}
Procedure DessinePolygone (X, Y : TableauReel; Lim : Integer; Mode: String);

Begin
     Couleur (Blanc);
     Polygone (X, Y, Lim, Mode);
End;

{===========================================================================}
Procedure EffacePolygone (X, Y : TableauReel; Lim : Integer; Mode : String);

Begin
     Couleur (Noir);
     Polygone (X, Y, Lim, Mode)
End;

{===========================================================================}
Procedure TraceCercle (XC, YC, R : Real; Pointille : Boolean);

Var     S, C, X, Y, Aux : Real;
        N               : Integer;

Begin
     S := Sin(Pi / 18);
     C := Cos(Pi / 18);
     X := XC + R;
     Y := YC;
       DeplaceEn (X, Y);
       For N := 2 to 37 Do
       Begin
            Aux := XC + (X - XC) * C - (Y - YC) * S;
            Y   := YC + (X - XC) * S + (Y - YC) * C;
            X   := Aux;
            if Pointille Then  DeplaceEn (X, Y);
            TraceVers (X, Y);
       End;
End;

{===========================================================================}
Procedure Cercle (XC, YC, R : Real);

Begin
     TraceCercle (XC, YC, R, False)
End;

{===========================================================================}
Procedure CerclePointille (XC, YC, R: real);

Begin
     TraceCercle (XC, YC, R, True);
End;

{===========================================================================}
Procedure Attente;

Var   Ch : Char;

Begin
    Sound(400) ;
    Delay (100) ;
    NoSound;
    Repeat Until KeyPressed;
    Ch := ReadKey
End;

{===========================================================================}
Procedure Bip;

Begin
    Sound(2000);
    Delay(50);
    NoSound;
End;

{===========================================================================}
Function Signe (N : Real): Real;

Begin
     if N >= 0 Then Signe := 1
                             Else Signe := -1;
End;

{===========================================================================}
Function Puissance (Base, Expo : Real ) : Real ;

Begin
    if Abs (Base) < 1E-6 Then
       if Expo = 0.0 Then
                         Puissance := 1.0
            Else Puissance := 0.0
    Else if Base > 0 Then
                   Puissance := Exp (Expo * Ln (Base))
            Else if Abs (Expo) >= 1 Then
       if Odd (Round (Expo)) Then
                   Puissance := Signe (Base)* Exp (Expo*Ln(Abs(Base)))
            Else Puissance := -Signe (Base) * Exp (Expo*Ln(Abs(Base)))
         Else Puissance := Signe (Base) * Exp (Expo*Ln(Abs(Base)));
End;

{===========================================================================}
Function Factorielle (X: integer) : Real;

Var     i : Integer;
        F : Real;

Begin
     if X < 2 Then
                Factorielle := 1
       Else
           Begin
                For i := 2 To X Do   F := F * i;
                Factorielle := F
           End;
End;

{***************************************************************************}
END.

